require "b2access"
-- Include Simple Tiled Implementation into project
local sti = require "sti"
inspect = require 'inspect'

io.stdout:setvbuf("no")

local debugDraw = true

function love.load()
	-- Set world meter size (in pixels)
	love.physics.setMeter(32)
	world = love.physics.newWorld(0, 0)
	map = sti("test.lua", { "box2d" })

	-- Prepare collision objects
	map:box2d_init(world)

	local groundBody
	for _, collisionObject in pairs(map.box2d_collision) do
		if collisionObject.object then
			if collisionObject.object.type == "Ground" then
				groundBody = collisionObject.body
			end
		else
		end
	end

	for _, collisionObject in pairs(map.box2d_collision) do
		if collisionObject.object then
			if collisionObject.object.properties.moveable then
				local world_x, world_y = collisionObject.body:getWorldCenter()
				local m_joint = love.physics.newFrictionJoint( groundBody, collisionObject.body, world_x, world_y, true )
				m_joint:setMaxForce( 150)
  				m_joint:setMaxTorque( 2000)
			end
		end
	end

	world:setCallbacks(beginContact, endContact, preSolve, postSolve)

	mousejoint = nil
end

function love.update(dt)
	local mouse_x, mouse_y = love.mouse.getPosition()
  	if mousejoint then mousejoint:setTarget(mouse_x, mouse_y) end

	map:update(dt)
	world:update(dt)
end

function love.draw()

	love.graphics.setColor(1, 1, 1)
	-- Draw world; transform and scale
	map:draw()
	
	if debugDraw then
		love.graphics.push()
		love.graphics.setColor(1, 0, 0)
		love.graphics.setLineWidth( 3)
		for _, body in pairs(world:getBodies()) do
		    for _, fixture in pairs(body:getFixtures()) do
		        local shape = fixture:getShape()
		        if shape:typeOf("CircleShape") then
		            local cx, cy = body:getWorldPoints(shape:getPoint())
		            love.graphics.circle("line", cx, cy, shape:getRadius())
		        elseif shape:typeOf("PolygonShape") then
		            love.graphics.polygon("line", body:getWorldPoints(shape:getPoints()))
		        else
		            love.graphics.line(body:getWorldPoints(shape:getPoints()))
		        end
		    end
		end
		love.graphics.pop()
	end
end

function love.mousepressed( x, y, button, istouch, presses )
	if button == 1 then
	    -- test na jaky objekt jsem klikl
	    for _, v1 in pairs(world:getBodies()) do
	      if v1:getType() == 'dynamic' then
	        local body_x, body_y = v1:getPosition()
	        local body_angle = v1:getAngle()
	        for _, v2 in pairs(v1:getFixtures()) do
	           local shape = v2:getShape()
	           local hit = shape:testPoint( body_x, body_y, body_angle, x, y );
	           if hit == true and not mousejoint then
	              --print(v2:getUserData())
	              mousejoint = world:newMouseJoint(v1, x, y)
	              mousejoint:setFrequency(5.0);
	              mousejoint:setDampingRatio(0.9);
	           end    
	        end
	      end
	    end
	  end

	local tilex, tiley = map:convertPixelToTile(x,y)

	local floor_tile_x, floor_tile_y = math.floor(tilex), math.floor(tiley)
	local gid = map.layers[1].data[floor_tile_x+1][floor_tile_y+1].gid
	--print(x, y, tilex, tiley, math.floor(tilex), math.floor(tiley), gid)

	for _, instance in ipairs(map.tileInstances[gid]) do
		print(instance.x, instance.y, x, y, tilex, tiley, math.floor(tilex), math.floor(tiley), gid)
		if instance.x == x and instance.y == y then
			print(inspect(instance))
		end
	end
end

function love.mousereleased( x, y, button, istouch, presses )

	if mousejoint then
	    mousejoint:destroy()
	    mousejoint = nil
	  end
end
